/*

   mlook.c

   Look up words in a dictionary.

   Syntax: mlook <dictionary>

   The dictionary must be sorted, one word per line.
   Words are read from standard input.
   Lines beginning with each word are written to standard output.
   If no line beginning with a given word is found,
   a message is written to standard output.

   Stephen G. Simpson <simpson@math.psu.edu>

   Version 1.0, 11 Sept 1993
   hacked from look.c in ispell-4.0

   Version 1.1, 28 October 1993
   added fflush(stdout) to flush output after each lookup

*/

#include <stdio.h>
#include <sys/stat.h>

int
compare(line, word)
  unsigned char *line, *word;
{
  int x;
  while (1)
    {
      if (*word == 0)
	return(0);
      if (*line == 0)
	return (1);
      x = *line++ - *word++;
      if (x != 0)
	return(x);
    }
}


int
main(argc, argv)
  int argc;
  char **argv;
{
  char *dict;
  unsigned char word[15];
  unsigned char buf[100];
  FILE *f;
  struct stat statb;
  long start, end, pos;
  int c, x, found;

  if (argc < 2)
  {
   printf("\nmlook: a command to look up words in a dictionary.\n\n");
   printf("Syntax: mlook <dictionary>\n\n");
   printf("The dictionary must be sorted, one word per line.\n");
   printf("Words are read from standard input.\n");
   printf("Lines beginning with each word are written to standard output.\n");
   printf("If no line beginning with a given word is found,\n");
   printf("a message is written to standard output.\n\n");
   exit (1);
   }

  if (argc > 2)
    {
      fprintf(stderr, "Syntax: mlook <dictionary>\n");
      exit (1);
    }

  dict = argv[1];

  if ((f = fopen(dict, "r")) == NULL)
    {
      fprintf(stderr, "mlook: can't open dictionary: %s\n", dict);
      exit (1);
    }

  if (stat(dict, &statb) < 0)
    {
      fprintf(stderr, "mlook: can't stat dictionary: %s\n", dict);
      exit(1);
    }

  start = 0;
  end = statb.st_size;

while (scanf ("%s", word) != EOF)
{
  fseek(f, (end - start) / 2, 0);
  while ((c = getc (f)) != EOF)
    if (c == '\n')
      break;

  /* first, find last word before the one we want */
  while (1)
    {
      pos = ftell(f);
      fgets(buf, sizeof buf, f);
      if (compare(buf, word) < 0)
	start = pos;
      else
	end = pos;

      pos = start + (end - start) / 2 - 100;
      if (pos <= start)
	break;
      fseek(f, pos, 0);
      while ((c = getc (f)) != EOF)
	if (c == '\n')
	  break;
      if (c == EOF)
	break;
    }
  if (pos < 0)
    pos = 0;
  fseek(f, pos, 0);

  /*
   * If we are at the file start, then don't skip to the next newline,
   * because then we would never match the first word in the file.
   */
  if (pos > 0)
    while ((c = getc (f)) != EOF)
      if (c == '\n')
        break;

   /* finally, output the matches */
  found = 0;
  while (fgets(buf, sizeof buf, f) != NULL)
    {
      x = compare(buf, word);
      if (x > 0)
           break;
      if (x == 0)
        {
           printf("%s", buf);
           found = 1;
        }
    }
  if (found == 0)
    printf("%s: no information available\n", word);
  fflush(stdout);
}
fclose(f);
return(0);
}
